window.Gleam.initSearch([{"doc":"mist","title":"mist","content":"# mist\n\n[![Package Version](https://img.shields.io/hexpm/v/mist)](https://hex.pm/packages/mist)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/mist/)\n\n## A glistening Gleam web server.\n\nTo follow along with the example below, you can create a new project and add the\ndependencies as follows:\n\n```sh\n$ gleam new &lt;your_project&gt;\n$ cd &lt;your_project&gt;\n$ gleam add mist gleam_erlang gleam_http gleam_otp\n```\n\nThe main entrypoints for your application are `mist.start_http` and\n`mist.start_https`. The argument to these functions is generated from the\nopaque `Builder` type. It can be constructed with the `mist.new` function, and\nfed updated configuration options with the associated methods (demonstrated\nin the examples below).\n\n```gleam\nimport gleam/bytes_builder\nimport gleam/erlang/process\nimport gleam/http/request.{type Request}\nimport gleam/http/response.{type Response}\nimport gleam/io\nimport gleam/iterator\nimport gleam/option.{None, Some}\nimport gleam/otp/actor\nimport gleam/result\nimport gleam/string\nimport mist.{type Connection, type ResponseData}\n\npub fn main() {\n  // These values are for the Websocket process initialized below\n  let selector = process.new_selector()\n  let state = Nil\n\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n\n  let assert Ok(_) =\n    fn(req: Request(Connection)) -&gt; Response(ResponseData) {\n      case request.path_segments(req) {\n        [&quot;ws&quot;] -&gt;\n          mist.websocket(\n            request: req,\n            on_init: fn(_conn) { #(state, Some(selector)) },\n            on_close: fn(_state) { io.println(&quot;goodbye!&quot;) },\n            handler: handle_ws_message,\n          )\n        [&quot;echo&quot;] -&gt; echo_body(req)\n        [&quot;chunk&quot;] -&gt; serve_chunk(req)\n        [&quot;file&quot;, ..rest] -&gt; serve_file(req, rest)\n        [&quot;form&quot;] -&gt; handle_form(req)\n\n        _ -&gt; not_found\n      }\n    }\n    |&gt; mist.new\n    |&gt; mist.port(3000)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n\npub type MyMessage {\n  Broadcast(String)\n}\n\nfn handle_ws_message(state, conn, message) {\n  case message {\n    mist.Text(&quot;ping&quot;) -&gt; {\n      let assert Ok(_) = mist.send_text_frame(conn, &quot;pong&quot;)\n      actor.continue(state)\n    }\n    mist.Text(_) | mist.Binary(_) -&gt; {\n      actor.continue(state)\n    }\n    mist.Custom(Broadcast(text)) -&gt; {\n      let assert Ok(_) = mist.send_text_frame(conn, text)\n      actor.continue(state)\n    }\n    mist.Closed | mist.Shutdown -&gt; actor.Stop(process.Normal)\n  }\n}\n\nfn echo_body(request: Request(Connection)) -&gt; Response(ResponseData) {\n  let content_type =\n    request\n    |&gt; request.get_header(&quot;content-type&quot;)\n    |&gt; result.unwrap(&quot;text/plain&quot;)\n\n  mist.read_body(request, 1024 * 1024 * 10)\n  |&gt; result.map(fn(req) {\n    response.new(200)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.from_bit_array(req.body)))\n    |&gt; response.set_header(&quot;content-type&quot;, content_type)\n  })\n  |&gt; result.lazy_unwrap(fn() {\n    response.new(400)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  })\n}\n\nfn serve_chunk(_request: Request(Connection)) -&gt; Response(ResponseData) {\n  let iter =\n    [&quot;one&quot;, &quot;two&quot;, &quot;three&quot;]\n    |&gt; iterator.from_list\n    |&gt; iterator.map(bytes_builder.from_string)\n\n  response.new(200)\n  |&gt; response.set_body(mist.Chunked(iter))\n  |&gt; response.set_header(&quot;content-type&quot;, &quot;text/plain&quot;)\n}\n\nfn serve_file(\n  _req: Request(Connection),\n  path: List(String),\n) -&gt; Response(ResponseData) {\n  let file_path = string.join(path, &quot;/&quot;)\n\n  // Omitting validation for brevity\n  mist.send_file(file_path, offset: 0, limit: None)\n  |&gt; result.map(fn(file) {\n    let content_type = guess_content_type(file_path)\n    response.new(200)\n    |&gt; response.prepend_header(&quot;content-type&quot;, content_type)\n    |&gt; response.set_body(file)\n  })\n  |&gt; result.lazy_unwrap(fn() {\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  })\n}\n\nfn handle_form(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let _req = mist.read_body(req, 1024 * 1024 * 30)\n  response.new(200)\n  |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n}\n\nfn guess_content_type(_path: String) -&gt; String {\n  &quot;application/octet-stream&quot;\n}\n```\n\n## Streaming request body\n\n```\nNOTE:  This is a new feature, and I may have made some mistakes.  Please let me\nknow if you run into anything :)\n```\n\nWhen handling file uploads or `multipart/form-data`, you probably don&#39;t want to\nload the whole file into memory. Previously, the only options in `mist` were to\naccept bodies up to `N` bytes, or read the entire body.\n\nNow, there is a `mist.stream` function which takes a `Request(Connection)` that\ngives you back a function to start reading chunks. This function will return:\n\n```gleam\npub type Chunk {\n  Chunk(data: BitString, consume: fn(Int) -&gt; Chunk)\n  Done\n}\n```\n\nNOTE: You must only call this once on the `Request(Connection)`. Since it&#39;s\nreading data from the socket, this is a mutable action. The name `consume` was\nchosen to hopefully make that more clear.\n\n### Example\n\n```gleam\n// Replacing the named function in the application example above\nfn handle_form(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let assert Ok(consume) = mist.stream(req)\n  // NOTE:  This is a little misleading, since `Iterator`s can be replayed.\n  // However, this will only be running this once.\n  let content =\n    iterator.unfold(\n      consume,\n      fn(consume) {\n        // Reads up to 1024 bytes from the request\n        let res = consume(1024)\n        case res {\n          // The error will not be bubbled up to the iterator here. If either\n          // we&#39;ve read all the body, or we see an error, the iterator finishes\n          Ok(mist.Done) | Error(_) -&gt; iterator.Done\n          // We read some data. It may be less than the specific amount above if\n          // we have consumed all of the body. You&#39;ll still need to call it\n          // again to ensure, since with `chunked` encoding, we need to check\n          // for the last chunk.\n          Ok(mist.Chunk(data, consume)) -&gt; {\n            iterator.Next(bit_builder.from_bit_string(data), consume)\n          }\n        }\n      },\n    )\n  // For fun, respond with `chunked` encoding of the same iterator\n  response.new(200)\n  |&gt; response.set_body(mist.Chunked(content))\n}\n```\n","url":"index.html"},{"doc":"mist","title":"Builder","content":"pub opaque type Builder(request_body, response_body)\n\n\nSynonyms:\nmist.Builder\nmist Builder","url":"mist.html#Builder"},{"doc":"mist","title":"CertificateError","content":"pub type CertificateError {\n  NoCertificate\n  NoKey\n  NoKeyOrCertificate\n}\n These are the types of errors raised by trying to read the certificate and\n key files.\nNoCertificate\n\n\nNoKey\n\n\nNoKeyOrCertificate\n\n\nSynonyms:\nmist.CertificateError\nmist CertificateError","url":"mist.html#CertificateError"},{"doc":"mist","title":"Chunk","content":"pub type Chunk {\n  Chunk(\n    data: BitArray,\n    consume: fn(Int) -&gt; Result(Chunk, ReadError),\n  )\n  Done\n}\n The values returning from streaming the request body. The `Chunk`\n variant gives back some data and the next token. `Done` signifies\n that we have completed reading the body.\nChunk(\n  data: BitArray,\n  consume: fn(Int) -&gt; Result(Chunk, ReadError),\n)\n\n\nDone\n\n\nSynonyms:\nmist.Chunk\nmist Chunk","url":"mist.html#Chunk"},{"doc":"mist","title":"Connection","content":"pub type Connection =\n  InternalConnection\n Re-exported type that represents the default `Request` body type. See\n `mist.read_body` to convert this type into a `BitString`. The `Connection`\n also holds some additional information about the request. Currently, the\n only useful field is `client_ip` which is a `Result` with a tuple of\n integers representing the IPv4 address.\n\nSynonyms:\nmist.Connection\nmist Connection","url":"mist.html#Connection"},{"doc":"mist","title":"ConnectionInfo","content":"pub type ConnectionInfo {\n  ConnectionInfo(port: Int, ip_address: IpAddress)\n}\n\nConnectionInfo(port: Int, ip_address: IpAddress)\n\n\nSynonyms:\nmist.ConnectionInfo\nmist ConnectionInfo","url":"mist.html#ConnectionInfo"},{"doc":"mist","title":"FileError","content":"pub type FileError {\n  IsDir\n  NoAccess\n  NoEntry\n  UnknownFileError\n}\n Potential errors when opening a file to send. This list is\n currently not exhaustive with POSIX errors.\nIsDir\n\n\nNoAccess\n\n\nNoEntry\n\n\nUnknownFileError\n\n\nSynonyms:\nmist.FileError\nmist FileError","url":"mist.html#FileError"},{"doc":"mist","title":"HttpsError","content":"pub type HttpsError {\n  GlistenError(glisten.StartError)\n  CertificateError(CertificateError)\n}\n These are the possible errors raised when trying to start an Https server.\n If there are issues reading the certificate or key files, those will be\n returned.\nGlistenError(glisten.StartError)\n\n\nCertificateError(CertificateError)\n\n\nSynonyms:\nmist.HttpsError\nmist HttpsError","url":"mist.html#HttpsError"},{"doc":"mist","title":"IpAddress","content":"pub type IpAddress {\n  IpV4(Int, Int, Int, Int)\n  IpV6(Int, Int, Int, Int, Int, Int, Int, Int)\n}\n When accessing client information, these are the possible shapes of the IP\n addresses. A best effort will be made to determine whether IPv4 is most\n relevant.\nIpV4(Int, Int, Int, Int)\n\n\nIpV6(Int, Int, Int, Int, Int, Int, Int, Int)\n\n\nSynonyms:\nmist.IpAddress\nmist IpAddress","url":"mist.html#IpAddress"},{"doc":"mist","title":"Port","content":"pub type Port {\n  Assigned\n  Provided(Int)\n}\n\nAssigned\n\n\nProvided(Int)\n\n\nSynonyms:\nmist.Port\nmist Port","url":"mist.html#Port"},{"doc":"mist","title":"ReadError","content":"pub type ReadError {\n  ExcessBody\n  MalformedBody\n}\n The possible errors from reading the request body. If the size is larger\n than the provided value, `ExcessBody` is returned. If there is an error\n reading the body from the socket or the body is malformed (i.e a chunked\n request with invalid sizes), `MalformedBody` is returned.\nExcessBody\n\n\nMalformedBody\n\n\nSynonyms:\nmist.ReadError\nmist ReadError","url":"mist.html#ReadError"},{"doc":"mist","title":"ResponseData","content":"pub type ResponseData {\n  Websocket(Selector(ProcessDown))\n  Bytes(BytesTree)\n  Chunked(Yielder(BytesTree))\n  File(descriptor: file.FileDescriptor, offset: Int, length: Int)\n  ServerSentEvents(Selector(ProcessDown))\n}\n The response body type. This allows `mist` to handle these different cases\n for you. `Bytes` is the regular data return. `Websocket` will upgrade the\n socket to websockets, but should not be used directly. See the\n `mist.upgrade` function for usage. `Chunked` will use\n `Transfer-Encoding: chunked` to send an iterator in chunks. `File` will use\n Erlang&#39;s `sendfile` to more efficiently return a file to the client.\nWebsocket(Selector(ProcessDown))\n\n\nBytes(BytesTree)\n\n\nChunked(Yielder(BytesTree))\n\n\nFile(descriptor: file.FileDescriptor, offset: Int, length: Int)\n See `mist.send_file` to use this response type.\n\nServerSentEvents(Selector(ProcessDown))\n\n\nSynonyms:\nmist.ResponseData\nmist ResponseData","url":"mist.html#ResponseData"},{"doc":"mist","title":"SSEConnection","content":"pub opaque type SSEConnection\n\n\nSynonyms:\nmist.SSEConnection\nmist SSEConnection","url":"mist.html#SSEConnection"},{"doc":"mist","title":"SSEEvent","content":"pub opaque type SSEEvent\n\n\nSynonyms:\nmist.SSEEvent\nmist SSEEvent","url":"mist.html#SSEEvent"},{"doc":"mist","title":"Server","content":"pub opaque type Server\n\n\nSynonyms:\nmist.Server\nmist Server","url":"mist.html#Server"},{"doc":"mist","title":"WebsocketConnection","content":"pub type WebsocketConnection =\n  InternalWebsocketConnection\n\n\nSynonyms:\nmist.WebsocketConnection\nmist WebsocketConnection","url":"mist.html#WebsocketConnection"},{"doc":"mist","title":"WebsocketMessage","content":"pub type WebsocketMessage(custom) {\n  Text(String)\n  Binary(BitArray)\n  Closed\n  Shutdown\n  Custom(custom)\n}\n These are the types of messages that a websocket handler may receive.\nText(String)\n\n\nBinary(BitArray)\n\n\nClosed\n\n\nShutdown\n\n\nCustom(custom)\n\n\nSynonyms:\nmist.WebsocketMessage\nmist WebsocketMessage","url":"mist.html#WebsocketMessage"},{"doc":"mist","title":"after_start","content":"pub fn after_start(\n  builder: Builder(a, b),\n  after_start: fn(Int, Scheme, IpAddress) -&gt; Nil,\n) -&gt; Builder(a, b)\n Override the default function to be called after the service starts. The\n default is to log a message with the listening port.\nSynonyms:\nmist.after_start\nmist after_start","url":"mist.html#after_start"},{"doc":"mist","title":"bind","content":"pub fn bind(\n  builder: Builder(a, b),\n  interface: String,\n) -&gt; Builder(a, b)\n Specify an interface to listen on. This is a string that can have the\n following values: &quot;localhost&quot;, a valid IPv4 address (i.e. &quot;127.0.0.1&quot;), or\n a valid IPv6 address (i.e. &quot;::1&quot;). An invalid value will cause the\n application to crash.\nSynonyms:\nmist.bind\nmist bind","url":"mist.html#bind"},{"doc":"mist","title":"event","content":"pub fn event(data: StringTree) -&gt; SSEEvent\n\nSynonyms:\nmist.event\nmist event","url":"mist.html#event"},{"doc":"mist","title":"event_id","content":"pub fn event_id(event: SSEEvent, id: String) -&gt; SSEEvent\n\nSynonyms:\nmist.event_id\nmist event_id","url":"mist.html#event_id"},{"doc":"mist","title":"event_name","content":"pub fn event_name(event: SSEEvent, name: String) -&gt; SSEEvent\n\nSynonyms:\nmist.event_name\nmist event_name","url":"mist.html#event_name"},{"doc":"mist","title":"get_client_info","content":"pub fn get_client_info(\n  conn: Connection,\n) -&gt; Result(ConnectionInfo, Nil)\n Tries to get the IP address and port of a connected client.\nSynonyms:\nmist.get_client_info\nmist get_client_info","url":"mist.html#get_client_info"},{"doc":"mist","title":"get_port","content":"pub fn get_port(server: Server) -&gt; Int\n\nSynonyms:\nmist.get_port\nmist get_port","url":"mist.html#get_port"},{"doc":"mist","title":"get_supervisor","content":"pub fn get_supervisor(server: Server) -&gt; Subject(Message)\n\nSynonyms:\nmist.get_supervisor\nmist get_supervisor","url":"mist.html#get_supervisor"},{"doc":"mist","title":"ip_address_to_string","content":"pub fn ip_address_to_string(address: IpAddress) -&gt; String\n Convenience function for printing the `IpAddress` type. It will convert the\n IPv6 loopback to the short-hand `::1`.\nSynonyms:\nmist.ip_address_to_string\nmist ip_address_to_string","url":"mist.html#ip_address_to_string"},{"doc":"mist","title":"new","content":"pub fn new(\n  handler: fn(Request(a)) -&gt; Response(b),\n) -&gt; Builder(a, b)\n Create a new `mist` handler with a given function. The default port is\n 4000.\nSynonyms:\nmist.new\nmist new","url":"mist.html#new"},{"doc":"mist","title":"port","content":"pub fn port(builder: Builder(a, b), port: Int) -&gt; Builder(a, b)\n Assign a different listening port to the service.\nSynonyms:\nmist.port\nmist port","url":"mist.html#port"},{"doc":"mist","title":"read_body","content":"pub fn read_body(\n  req: Request(Connection),\n  max_body_limit max_body_limit: Int,\n) -&gt; Result(Request(BitArray), ReadError)\n The request body is not pulled from the socket until requested. The\n `content-length` header is used to determine whether the socket is read\n from or not. The read may also fail, and a `ReadError` is raised.\nSynonyms:\nmist.read_body\nmist read_body","url":"mist.html#read_body"},{"doc":"mist","title":"read_request_body","content":"pub fn read_request_body(\n  builder: Builder(BitArray, a),\n  bytes_limit bytes_limit: Int,\n  failure_response failure_response: Response(a),\n) -&gt; Builder(Connection, a)\n This function allows for implicitly reading the body of requests up\n to a given size. If the size is too large, or the read fails, the provided\n `failure_response` will be sent back as the response.\nSynonyms:\nmist.read_request_body\nmist read_request_body","url":"mist.html#read_request_body"},{"doc":"mist","title":"send_binary_frame","content":"pub fn send_binary_frame(\n  connection: WebsocketConnection,\n  frame: BitArray,\n) -&gt; Result(Nil, SocketReason)\n Sends a binary frame across the websocket.\nSynonyms:\nmist.send_binary_frame\nmist send_binary_frame","url":"mist.html#send_binary_frame"},{"doc":"mist","title":"send_event","content":"pub fn send_event(\n  conn: SSEConnection,\n  event: SSEEvent,\n) -&gt; Result(Nil, Nil)\n\nSynonyms:\nmist.send_event\nmist send_event","url":"mist.html#send_event"},{"doc":"mist","title":"send_file","content":"pub fn send_file(\n  path: String,\n  offset offset: Int,\n  limit limit: Option(Int),\n) -&gt; Result(ResponseData, FileError)\n To respond with a file using Erlang&#39;s `sendfile`, use this function\n with the specified offset and limit (optional). It will attempt to open the\n file for reading, get its file size, and then send the file.  If the read\n errors, this will return the relevant `FileError`. Generally, this will be\n more memory efficient than manually doing this process with `mist.Bytes`.\nSynonyms:\nmist.send_file\nmist send_file","url":"mist.html#send_file"},{"doc":"mist","title":"send_text_frame","content":"pub fn send_text_frame(\n  connection: WebsocketConnection,\n  frame: String,\n) -&gt; Result(Nil, SocketReason)\n Sends a text frame across the websocket.\nSynonyms:\nmist.send_text_frame\nmist send_text_frame","url":"mist.html#send_text_frame"},{"doc":"mist","title":"server_sent_events","content":"pub fn server_sent_events(\n  request req: Request(Connection),\n  initial_response resp: Response(a),\n  init init: fn() -&gt; InitResult(b, c),\n  loop loop: fn(c, SSEConnection, b) -&gt; Next(c, b),\n) -&gt; Response(ResponseData)\n Sets up the connection for server-sent events. The initial response provided\n here will have its headers included in the SSE setup. The body is discarded.\n The `init` and `loop` parameters follow the same shape as the\n `gleam/otp/actor` module.\n\n NOTE:  There is no proper way within the spec for the server to &quot;close&quot; the\n SSE connection. There are ways around it.\n\n See:  `examples/eventz` for a sample usage.\nSynonyms:\nmist.server_sent_events\nmist server_sent_events","url":"mist.html#server_sent_events"},{"doc":"mist","title":"start_http","content":"pub fn start_http(\n  builder: Builder(Connection, ResponseData),\n) -&gt; Result(Subject(Message), StartError)\n Start a `mist` service over HTTP with the provided builder.\nSynonyms:\nmist.start_http\nmist start_http","url":"mist.html#start_http"},{"doc":"mist","title":"start_http_server","content":"pub fn start_http_server(\n  builder: Builder(Connection, ResponseData),\n) -&gt; Result(Server, StartError)\n See the documentation for `start_http`.  For now, you almost certainly\n want to use that.  In the future, this will allow access to things like\n OS-provided ports, graceful shutdown, etc.\nSynonyms:\nmist.start_http_server\nmist start_http_server","url":"mist.html#start_http_server"},{"doc":"mist","title":"start_https","content":"pub fn start_https(\n  builder: Builder(Connection, ResponseData),\n  certfile certfile: String,\n  keyfile keyfile: String,\n) -&gt; Result(Subject(Message), HttpsError)\n Start a `mist` service over HTTPS with the provided builder. This method\n requires both a certificate file and a key file. The library will attempt\n to read these files off of the disk.\nSynonyms:\nmist.start_https\nmist start_https","url":"mist.html#start_https"},{"doc":"mist","title":"start_https_server","content":"pub fn start_https_server(\n  builder: Builder(Connection, ResponseData),\n  certfile certfile: String,\n  keyfile keyfile: String,\n) -&gt; Result(Server, HttpsError)\n See the documentation for `start_https`.  For now, you almost certainly\n want to use that.  In the future, this will allow access to things like\n OS-provided ports, graceful shutdown, etc.\nSynonyms:\nmist.start_https_server\nmist start_https_server","url":"mist.html#start_https_server"},{"doc":"mist","title":"stream","content":"pub fn stream(\n  req: Request(Connection),\n) -&gt; Result(fn(Int) -&gt; Result(Chunk, ReadError), ReadError)\n Rather than explicitly reading either the whole body (optionally up to\n `N` bytes), this function allows you to consume a stream of the request\n body. Any errors reading the body will propagate out, or `Chunk`s will be\n emitted. This provides a `consume` method to attempt to grab the next\n `size` chunk from the socket.\nSynonyms:\nmist.stream\nmist stream","url":"mist.html#stream"},{"doc":"mist","title":"websocket","content":"pub fn websocket(\n  request request: Request(Connection),\n  handler handler: fn(a, WebsocketConnection, WebsocketMessage(b)) -&gt;\n    Next(b, a),\n  on_init on_init: fn(WebsocketConnection) -&gt;\n    #(a, Option(Selector(b))),\n  on_close on_close: fn(a) -&gt; Nil,\n) -&gt; Response(ResponseData)\n Upgrade a request to handle websockets. If the request is\n malformed, or the websocket process fails to initialize, an empty\n 400 response will be sent to the client.\n\n The `on_init` method will be called when the actual WebSocket process\n is started, and the return value is the initial state and an optional\n selector for receiving user messages.\n\n The `on_close` method is called when the WebSocket process shuts down\n for any reason, valid or otherwise.\nSynonyms:\nmist.websocket\nmist websocket","url":"mist.html#websocket"},{"doc":"mist","title":"with_ipv6","content":"pub fn with_ipv6(builder: Builder(a, b)) -&gt; Builder(a, b)\n By default, `mist` will listen on `localhost` over IPv4. If you specify an\n IPv4 address to bind to, it will still only serve over IPv4. Calling this\n function will listen on both IPv4 and IPv6 for the given interface. If it is\n not supported, your application will crash. If you provide an IPv6 address\n to `mist.bind`, this function will have no effect.\nSynonyms:\nmist.with_ipv6\nmist with_ipv6","url":"mist.html#with_ipv6"},{"doc":"mist","title":"mist","content":"","url":"mist.html"}]);